"use strict";

define([
    "jquery",
    "loglevel",
    "./notificationmanager",
    "./featurehandler",
    "./icons",
    "./utils"
],
function ($, loglevel, notificationManager, featurehandler, icons, utils) {
    var log,
        connection,
        fileExplorer,
        $fileTableRow,
        backFolder,
        browseFromCurrent,
        listing = {
            name: "",
            elements: []
        },
        downloadNotificationManager,
        uploadNotificationManager,
        filestorageNotificationManager,
        dateFormatter,
        downloadEnabled = false,
        uploadEnabled = false;

    function changeDir (path) {
        var dir, cmd;

        dir = path || "";

        log.debug("[File Storage UI] Changing to directory " + dir);

        /* Avoid invalid fullpaths starting with multiple '/' */
        if (dir === "/") {
            dir = "";
        }

        listing.name = dir;
        listing.elements = [];

        cmd = fileExplorer.list(listing.name);
        cmd.on("error", function (error) {
            filestorageNotificationManager.addNotification({
                level: notificationManager.level.ERROR,
                text: "Failed to list files (" + error.message + ")",
                actions: [{
                    dismissable: true
                }]
            });
            if (dir !== "" || dir !== "/") {
                changeDir("");
            }
        }).on("success", function () {
            log.debug("[File Storage UI] List files succeeded");
            updateListFolders();
        }).on("newItems", function (items) {
            log.debug("[File Storage UI] New items " + items.length);

            // FIXME: show the items as we get them
            listing.elements = listing.elements.concat(items);
        }).run();
    }

    function showDir () {
        if (browseFromCurrent) {
            changeDir(listing.name);
        } else {
            changeDir();
        }
    }

    function onModalHide () {
        $("#dcv-filestorage-table-tbody").empty();
        $("#dcv-filestorage-table-tbody").append($fileTableRow);
        connection.setUploadBandwidth(null);
        browseFromCurrent = false;
    }

    function onModalShow () {
        connection.getFileExplorer().then(function (explorer) {
            if (!$fileTableRow) {
                $fileTableRow = $("#fileTableRow").clone(true);
            }
            fileExplorer = explorer;
            showDir();
            connection.setUploadBandwidth(50 * 1024);
        }).catch(function (error) {
            $("#dcv-filestorage-table-thead").hide();
            $("#dcv-filestorage-table-tbody").empty();
            $("#dcv-filestorage-table-tbody").append("<tr class=\"dcv-filestorage-empty\"><td>File storage is not available</td></tr>");
        });
    }

    function onModalShown () {
        if ($("#inputFileStorageText")) {
            $("#inputFileStorageText").focus();
        }
    }

    function mkDir (dir) {
        var cmd;

        cmd = fileExplorer.createDirectory(dir);
        cmd.on("error", function (error) {
            filestorageNotificationManager.addNotification({
                level: notificationManager.level.ERROR,
                text: "Failed to create directory (" + error.message + ")",
                actions: [{
                    dismissable: true
                }]
            });
            updateListFolders();
        }).on("success", function () {
            log.debug("[File Storage UI] Mkdir succeeded");
            browseFromCurrent = true;
            showDir();
        }).run();
    }

    function createFolder () {
        var $createRowToAppend,
            inputFolderName;

        if (listing.elements.length === 0) {
            /* Show file header and remove empty message */
            $("#dcv-filestorage-table-thead").show();
            $("#dcv-filestorage-table-tbody").empty();
        }

        $createRowToAppend = $fileTableRow.clone(true);
        inputFolderName = "<input id=\"inputFileStorageText\" type=\"text\" value=\"New Folder\" size=\"30\" maxlength=\"80\" tabindex=\"20\" />";
        $createRowToAppend.prop("id", "NewFolderInsertName");
        $createRowToAppend.find("#dcv-filestorage-file-icon").html(icons.storage);
        $createRowToAppend.find("#dcv-filestorage-file-name").html(inputFolderName);
        $createRowToAppend.find("#dcv-filestorage-file-date").html(" ");
        $createRowToAppend.find("#dcv-filestorage-file-size").html(" ");
        $createRowToAppend.find("#dcv-filestorage-file-action").html(" ");
        $createRowToAppend.prependTo($("#dcv-filestorage-table-tbody"));

        $("#inputFileStorageText").focus();
        $("#inputFileStorageText").on("blur", function () {
            $("#inputFileStorageText").off("blur");
            if ($("#inputFileStorageText").prop("value").length > 0) {
                mkDir(listing.name + "/" + $("#inputFileStorageText").prop("value"));
                $("#inputFileStorageText").disabled = true; // disabled input, waiting for command result
            } else {
                updateListFolders();
            }
        });
        $("#inputFileStorageText").on("keyup", function (e) {
            if (e.keyCode === 13) {
                /*
                 * Edge and IE11 stops reacting to events if you only call rename here.
                 * You must force a blur event on the input element to prevent issues.
                 * The rename event will be called by the previous on-blur callback.
                 */

                $("#inputFileStorageText").blur();
            }
        });
    }

    /* Temporary delete removal until Undo action is implemented
    function remove (dest) {
        var cmd;

        cmd = fileExplorer.deleteFile(dest);
        cmd.on("error", function (error) {
            alert("Failed to remove (" + error.message + ")");
        }).on("success", function () {
            log.debug("Remove succeeded");
            browseFromCurrent = true;
            showDir();
        }).run();
    }
    */

    function rename (src, dest, folder) {
        var cmd;

        if (folder) {
            cmd = fileExplorer.renameDirectory(src, dest);
        } else {
            cmd = fileExplorer.renameFile(src, dest);
        }
        cmd.on("error", function (error) {
            filestorageNotificationManager.addNotification({
                level: notificationManager.level.ERROR,
                text: "Failed to rename (" + error.message + ")",
                actions: [{
                    dismissable: true
                }]
            });
            updateListFolders();
        }).on("success", function () {
            log.debug("[File Storage UI] Rename succeeded");
            browseFromCurrent = true;
            showDir();
        }).run();
    }

    function renameTarget (target, isFolder, id) {
        var $rowToRename,
            idString,
            newFileName,
            inputRename;

        if (isFolder) {
            idString = "#Folder-" + id;
        } else {
            idString = "#File-" + id;
        }
        $rowToRename = $("#dcv-filestorage-table-tbody").find(idString);
        inputRename = "<input id=\"inputFileStorageText\" type=\"text\" value=\"" + target.displayName + "\" size=\"30\" maxlength=\"80\" tabindex=\"20\" />";
        $rowToRename.find("#dcv-filestorage-file-name").html(inputRename);
        $rowToRename.find("#dcv-filestorage-actions").html("");

        $("#inputFileStorageText").focus();
        $("#inputFileStorageText").on("blur", function () {
            $("#inputFileStorageText").off("blur");
            newFileName = $("#inputFileStorageText").prop("value");
            if ((newFileName.length > 0) && (target.displayName !== newFileName)) {
                rename(listing.name + "/" + target.path, listing.name + "/" + newFileName, isFolder);
            } else {
                updateListFolders();
            }
        });
        $("#inputFileStorageText").on("keyup", function (e) {
            if (e.keyCode === 13) {
                /*
                 * Edge and IE11 stops reacting to events if you only call rename here.
                 * You must force a blur event on the input element to prevent issues.
                 * The rename event will be called by the previous on-blur callback.
                 */

                $("#inputFileStorageText").blur();
            }
        });
    }

    function download (path) {
        var cmd;

        cmd = fileExplorer.retrieveFile(path);
        cmd.on("error", function (error) {
            downloadNotificationManager.addNotification({
                level: notificationManager.level.ERROR,
                text: "Impossible to download resource " + path + "<br />Error message: " + error.message + " (code: " + error.code + ")",
                actions: [{
                    dismissable: true
                }]
            });
        }).on("success", function () {
            downloadNotificationManager.addNotification({
                level: notificationManager.level.INFO,
                text: "File " + path + " ready to be downloaded in your device.",
                actions: [{
                    dismissable: true
                }]
            });
        }).run();
    }

    function upload (file, dir) {
        var cmd,
            notification;

        cmd = fileExplorer.storeFile(file, dir);
        notification = uploadNotificationManager.addNotification({
            level: notificationManager.level.INFO,
            text: file.name + " is uploading to Storage" + dir,
            progress: 0,
            actions: [{
                text: "Stop uploading.",
                callback: clickedUploadCancel.bind(null, cmd),
                remove: true
            }]
        });
        cmd.on("error", function (oldNotification, error) { // the first argument comes from bind, the others from the apply call
            uploadNotificationManager.addNotification({
                level: notificationManager.level.ERROR,
                text: file.name + " upload to Storage" + dir + " failed!<br />Error: " + error.message + " (code: " + error.code + ")",
                actions: [{
                    dismissable: true
                }]
            }, oldNotification);
        }.bind(null, notification)).on("progress", function (oldNotification, partial, total) { // the first argument comes from bind, the others from the apply call
            arguments[0].updateProgress(Math.floor(100 * partial / total));
        }.bind(null, notification)).on("success", function () {
            uploadNotificationManager.addNotification({
                level: notificationManager.level.INFO,
                text: file.name + " uploaded to Storage" + dir,
                actions: [{
                    dismissable: true
                }]
            }, arguments[0]);
            browseFromCurrent = true;
            showDir();
        }.bind(null, notification)).run();
    }

    function clickedUploadCancel () {
        var cmd = arguments[0];

        cmd.cancel();
    }

    function updateHeader () {
        var pathElements, parentName, tooltipTitle;

        if (listing.name === "" || listing.name === "/") {
            $("#dcv-filestorage-back-button").hide();
            $("#dcv-filestorage-folder-name").html("Storage");
        } else {
            pathElements = listing.name.split("/");
            $("#dcv-filestorage-back-button").show();
            $("#dcv-filestorage-folder-name").html(pathElements.pop());
            parentName = pathElements.pop();
            if (parentName === null || parentName.length === 0 || parentName === "/") {
                tooltipTitle = "Go Back To Session Storage";
            } else {
                tooltipTitle = "Go Back To " + parentName;
            }

            backFolder = listing.name.substring(0, listing.name.lastIndexOf("/"));

            /* Replace tooltip's title */
            $("#dcv-filestorage-back-button").attr("title", tooltipTitle);
        }
    }

    /* eslint-disable no-loop-func*/
    /* eslint-disable no-invalid-this*/

    function updateListFolders () {
        var $rowToAppend,
            i,
            idString,
            elements,
            files = [],
            folders = [];

        log.debug("[File Storage UI] update list folders " + listing.name);

        updateHeader();

        $("#dcv-filestorage-table-tbody").empty();

        elements = listing.elements;
        if (elements && (elements.length > 0)) {
            elements.forEach(function (elem) {
                if (elem.type === "file") {
                    files.push(elem);
                }
                if (elem.type === "folder") {
                    folders.push(elem);
                }
            });

            $("#dcv-filestorage-table-thead").show();

            for (i = 0; i < folders.length; i++) {
                $rowToAppend = $fileTableRow.clone(true);
                idString = "Folder-" + i;
                $rowToAppend.prop("id", idString);
                $rowToAppend.find("#dcv-filestorage-file-icon").html("<a href='#' class='dcv-filestorage-action-default' title='Open'>" + icons.storage + "</a>");
                $rowToAppend.find("#dcv-filestorage-file-name").html("<a href='#' class='dcv-filestorage-action-default' title='Open'>" + folders[i].displayName + "</a>");
                $rowToAppend.find("#dcv-filestorage-file-date").html(dateFormatter.format(new Date(folders[i].lastModified)));
                $rowToAppend.find("#dcv-filestorage-file-size").html(" - ");

                $rowToAppend.find(".dcv-filestorage-action-default").on("click", function () {
                    changeDir(listing.name + "/" + this.path);
                }.bind(folders[i], i));
                $rowToAppend.find("#dcv-filestorage-action-open-folder").on("click", function () {
                    changeDir(listing.name + "/" + this.path);
                }.bind(folders[i], i));
                $rowToAppend.find("#dcv-filestorage-action-rename").on("click", function () {
                    log.debug("[File Storage UI] Renaming " + listing.name + "/" + this.path);
                    renameTarget(this, true, arguments[0]);
                }.bind(folders[i], i));

                $rowToAppend.find("#dcv-filestorage-action-delete").remove();

                /* Temporary delete removal until Undo action is implemented
                $rowToAppend.find("#dcv-filestorage-action-delete").on("click", function () {
                    log.debug("[File Storage UI] Deleting " + listing.name + "/" + this.path);
                    remove(listing.name + "/" + this.path);
                }.bind(folders[i]));
                */

                $rowToAppend.find("#dcv-filestorage-action-download").remove();

                $("#dcv-filestorage-table-tbody").append($rowToAppend);
            }
            for (i = 0; i < files.length; i++) {
                $rowToAppend = $fileTableRow.clone(true);
                idString = "File-" + i;
                $rowToAppend.prop("id", idString);
                $rowToAppend.find("#dcv-filestorage-file-icon").html("<a href='#' class='dcv-filestorage-action-default' title='Download'>" + icons.document + "</a>");
                $rowToAppend.find("#dcv-filestorage-file-name").html("<a href='#' class='dcv-filestorage-action-default' title='Download'>" + files[i].displayName + "</a>");
                $rowToAppend.find("#dcv-filestorage-file-date").html(dateFormatter.format(new Date(files[i].lastModified)));
                $rowToAppend.find("#dcv-filestorage-file-size").html(printableSize(files[i].size));

                if (downloadEnabled) {
                    $rowToAppend.find(".dcv-filestorage-action-default").on("click", function () {
                        log.debug("[File Storage UI] Downloading " + listing.name + "/" + this.path);
                        download(listing.name + "/" + this.path);
                    }.bind(files[i]));
                }
                $rowToAppend.find("#dcv-filestorage-action-open-folder").remove();
                $rowToAppend.find("#dcv-filestorage-action-rename").on("click", function () {
                    log.debug("[File Storage UI] Renaming " + listing.name + "/" + this.path);
                    renameTarget(this, false, arguments[0]);
                }.bind(files[i], i));

                $rowToAppend.find("#dcv-filestorage-action-delete").remove();

                /* Temporary delete removal until Undo action is implemented
                $rowToAppend.find("#dcv-filestorage-action-delete").on("click", function () {
                    log.debug("[File Storage UI] Deleting " + listing.name + "/" + this.path);
                    remove(listing.name + "/" + this.path);
                }.bind(files[i]));
                */

                if (downloadEnabled) {
                    $rowToAppend.find("#dcv-filestorage-action-download").on("click", function () {
                        log.debug("[File Storage UI] Downloading " + listing.name + "/" + this.path);
                        download(listing.name + "/" + this.path);
                    }.bind(files[i]));
                } else {
                    $rowToAppend.find("#dcv-filestorage-action-download").remove();
                }

                $("#dcv-filestorage-table-tbody").append($rowToAppend);
            }
        } else {
            $("#dcv-filestorage-table-thead").hide();
            $("#dcv-filestorage-table-tbody").append("<tr class=\"dcv-filestorage-empty\"><td>" + $("#dcv-filestorage-folder-name").html() + " is empty</td></tr>");
        }
    }

    /* eslint-enable no-invalid-this*/
    /* eslint-enable no-loop-func*/

    function printableSize (size) {
        var sizeValue;

        sizeValue = parseInt(size, 10);
        if (sizeValue < 1024) {
            return (sizeValue + "&nbsp;&nbsp;B");
        }
        sizeValue = Math.ceil(sizeValue / 1024);
        if (sizeValue < 1024) {
            return (sizeValue + "&nbsp;KB");
        }
        sizeValue = Math.ceil(sizeValue / 1024);
        if (sizeValue < 1024) {
            return (sizeValue + "&nbsp;MB");
        }
        sizeValue = Math.ceil(sizeValue / 1024);
        if (sizeValue < 1024) {
            return (sizeValue + "&nbsp;GB");
        }
        sizeValue = Math.ceil(sizeValue / 1024);
        if (sizeValue < 1024) {
            return (sizeValue + "&nbsp;TB");
        }
        sizeValue = Math.ceil(sizeValue / 1024);
        return (sizeValue + "&nbsp;PB");
    }

    function filestorageEnabled (status) {
        if (status.enabled) {
            $("#dcv-filestorage-button").show();
        } else {
            if ($("#dcv-filestorage").hasClass("in")) {
                $("#dcv-filestorage").modal("hide");
            }
            $("#dcv-filestorage-button").hide();
        }
    }

    function downloadEnable (status) {
        downloadEnabled = status.enabled;
    }

    function uploadEnable (status) {
        uploadEnabled = status.enabled;
        if (uploadEnabled) {
            $("#dcv-filestorage-upload-button").show();
            $("#dcv-filestorage-upload-button").on("click", function () {
                $("#fileUploadInput").click();
            });
        } else {
            $("#dcv-filestorage-upload-button").hide();
        }
    }

    return {
        setup: function (conn) {
            log = loglevel.getLogger("filestorage");

            connection = conn;
            downloadNotificationManager = notificationManager.getCategoryHandler("download");
            uploadNotificationManager = notificationManager.getCategoryHandler("upload");
            filestorageNotificationManager = notificationManager.getCategoryHandler("storage");
            featurehandler.addFeatureListener("file-download", downloadEnable);
            featurehandler.addFeatureListener("file-upload", uploadEnable);

            $("#dcv-filestorage").on("hide.bs.modal", onModalHide);
            $("#dcv-filestorage").on("show.bs.modal", onModalShow);
            $("#dcv-filestorage").on("shown.bs.modal", onModalShown);
            $("#dcv-filestorage").find("#firstRowInput").hide();

            $("#dcv-filestorage-back-button").on("click", function (e) {
                changeDir(backFolder);
            });
            $("#dcv-filestorage-create-folder-button").on("click", function () {
                createFolder();
            });

            /* eslint-disable no-invalid-this */

            $("#fileUploadInput").on("change", function (e) {
                var fileList = this.files;

                if (fileList && fileList.length > 0) {
                    log.debug("[File Storage UI] A file to upload has been selected.");
                    upload(fileList[0], listing.name);

                    // reset the input
                    $(this).val("");
                } else {
                    log.debug("[File Storage UI] No file to upload has been selected.");
                }
            });

            /* eslint-enable no-invalid-this */

            browseFromCurrent = false;

            dateFormatter = new Intl.DateTimeFormat("en-US", {
                year: "numeric", month: "numeric", day: "numeric",
                hour: "numeric", minute: "numeric",
                hour12: true
            });
        },

        enabled: function (enabled) {
            filestorageEnabled({enabled: enabled});
        },

        downloadResource: function (fileResource) {
            var formData;

            if (!downloadEnabled) {
                return;
            }

            formData = document.createElement("form");
            formData.setAttribute("id", "resourceForm");
            formData.setAttribute("action", fileResource.URL);
            formData.setAttribute("method", "post");
            formData.setAttribute("target", "_blank");
            $("#dcv-resource").append(formData);
            $("#resourceForm").append("<input type=\"hidden\" name=\"token\" value=\"" + fileResource.token + "\" />");
            $("#resourceForm").submit();
            $("#resourceForm").remove();
        }
    };
});

/* ex:set ts=4 et: */
