"use strict";

define([
    "jquery",
    "loglevel",
    "bowser",
    "./notificationmanager"
],
function ($, loglevel, bowser, notificationManager) {
    var log,
        printerNotifications;

    /* The following functions are just an example, in the UI more than one
     * resource must be handled.
     * URL contains the relative url of the printed file, eg /resourse/printer...
     * The function decline and the download URL need the base path, the baseURL
     * can be obtained using the function getBaseURL() or specified if different
     */
    function newPrintAvailable (resource) {
        /* We must check if the browser is Chrome or not. If yes, Chrome will
         * fail in saving the file after that is loaded in the PDF viewer since
         * it will try to download it again from the server. Thus, in the case
         * of Chrome we force the download action.
         * Chrome bug: https://bugs.chromium.org/p/chromium/issues/detail?id=69677
         */
        if (bowser.chrome) {
            printerNotifications.addNotification({
                level: notificationManager.level.INFO,
                text: resource.id + " has been converted in printable format.",
                actions: [{
                    text: "Download it.",
                    download: resource.URL + "&disposition=attachment",
                    remove: true
                }, {
                    text: "Decline it.",
                    callback: clickedPrintDecline.bind(null, resource),
                    remove: true
                }]
            });
        } else {
            printerNotifications.addNotification({
                level: notificationManager.level.INFO,
                text: resource.id + " has been converted in printable format.",
                actions: [{
                    text: "Show in a new tab.",
                    link: resource.URL,
                    remove: true
                }, {
                    text: "Decline it.",
                    callback: clickedPrintDecline.bind(null, resource),
                    remove: true
                }]
            });
        }
        log.debug("Added printer notification for resource " + resource.id);
    }

    function clickedPrintDecline () {
        var resource = arguments[0];

        log.debug("Removing print resource " + resource.id);
        resource.decline();
    }

    return {
        setup: function () {
            log = loglevel.getLogger("printer");

            printerNotifications = notificationManager.getCategoryHandler("printer");
        },

        handleResource: function (printResource) {
            newPrintAvailable(printResource);
        }
    };
});

/* ex:set ts=4 et: */
