"use strict";

define([
    "./utils"
],
function (utils) {
    var settings = {
        sessionId: "",
        authToken: "", // FIXME: this is not really a setting
        logLevel: "info",
        wss: true,
        enabledChannels: [],
        customChannels: [],
        displayCodecs: [],
        enableQU: true,
        minQuality: 30,
        maxQuality: 0,
        maxDisplayWidth: 1920,
        maxDisplayHeight: 1080,
        minDisplayWidth: 570,
        minDisplayHeight: 480,
        debugLayer: false,
        dynamicResolution: true,
        enableShortcutEditor: false,
        scaleToFit: false,
        volume: 100,
        audioBitrate: 0,
        fingerRadius: 20,
        microphone: false,
        resourceBaseUrl: "", // FIXME: this is not really a setting
        persist: function () {
            localStorage.setItem("dcv-settings", JSON.stringify(settings));
        },
        load: function () {
            var savedSettings = JSON.parse(localStorage.getItem("dcv-settings"));

            if (savedSettings) {
                // add here other settings that should be persisted
                settings.dynamicResolution = savedSettings.dynamicResolution;

                if (savedSettings.volume >= 0 && savedSettings.volume <= 100) {
                    settings.volume = savedSettings.volume;
                }
            }

            urlParamsSettingsOverride();
        }
    };

    // override settings from the query string
    function urlParamsSettingsOverride () {
        var urlParams;

        urlParams = utils.getUrlParams();
        Object.keys(urlParams).forEach(function (name) {
            if (name in settings) {
                if (name === "sessionId") {
                    console.log("Unsupported URL parameter sessionId. The session ID must specified in the url after #");
                } else {
                    console.log("Specified URL parameter " + name + " with value " + urlParams[name]);

                    if (Array.isArray(settings[name])) {
                        settings[name] = urlParams[name].split(",");
                    } else if (typeof settings[name] === "boolean") {
                        settings[name] = (urlParams[name].toLowerCase() === "true");
                    } else if (typeof settings[name] === "number") {
                        settings[name] = parseFloat(urlParams[name]);
                    } else {
                        settings[name] = decodeURIComponent(urlParams[name]);
                    }
                }
            } else {
                console.log("Unsupported URL parameter " + name);
            }
        });

        // this is an ugly special case, but in our limited use cases
        // we prefer to have 'undefined' instead of an empty array
        // since undefined evaluates to false while [] does not.
        Object.keys(settings).forEach(function (name) {
            if (Array.isArray(settings[name]) && settings[name].length === 0) {
                delete settings[name];
            }
        });

        // This is really not a setting, but this value comes from URL parsing
        settings.sessionId = utils.getUrlHash();

        if (utils.isUndefinedOrNull(settings.sessionId) || settings.sessionId === "") {
            console.log("No session ID was specified in the URL");
        } else {
            console.log("Specified URL session ID is " + settings.sessionId);
        }
    }

    return settings;
});

/* ex:set ts=4 et: */
