"use strict";

define(function () {

    return {
        isFunction: function (f) {
            return typeof f === "function";
        },

        getUrlParams: function () {
            var params = {};

            window.location.href.replace(
                /[?&]+([^=&]+)=([^&#]*)/gi,
                function (m, key, value) {
                    params[key] = value;
                });

            return params;
        },

        /* The following splits the hash in the URL, it filters every character
         * after a second #. Only the first one is returned or null if empty.
         * E.g. https://server:port/ will return ""
         * E.g. https://server:port/# will return ""
         * E.g. https://server:port/#pippo will return pippo
         * E.g. https://server:port/#pippo#pluto will return pippo
         */
        getUrlHash: function () {
            var params = window.location.hash.split("#");

            if (params.length < 2) {
                return "";
            }
            return params[1];
        },

        isUndefinedOrNull: function (val) {
            return ((typeof val === "undefined") || (val === null));
        },

        formatTraffic: function (bitsPerSec) {
            if (bitsPerSec < 1000) {
                return bitsPerSec.toFixed(1) + " bps";
            } else if (bitsPerSec < 1000 * 1000) {
                return (bitsPerSec / 1000).toFixed(1) + " kbps";
            }
            return (bitsPerSec / (1000 * 1000)).toFixed(1) + " Mbps";
        }
    };
});

/* ex:set ts=4 et: */
