/* eslint-disable new-cap */

"use strict";

define(["./sha1"], function (JSSHA) {

    var methods;

    function text2ua (s) {
        var i, ua;

        ua = new Uint8Array(s.length);
        for (i = 0; i < s.length; i++) {
            ua[i] = s.charCodeAt(i);
        }
        return ua;
    }

    function ua2text (ua) {
        return String.fromCharCode.apply(null, ua);
    }

    function xor (a, b) {
        var i, res;

        res = "";
        for (i = 0; i < Math.min(a.length, b.length); i++) {
            res += String.fromCharCode(a.charCodeAt(i) ^ b.charCodeAt(i));
        }

        return res;
    }

    methods = {
        NONCE: function (size) {
            var bytes, crypto;

            crypto = window.crypto || window.msCrypto;

            if (crypto && crypto.getRandomValues) {
                bytes = new Uint8Array(size);

                /* This will not work in older browsers.
                 * See https://developer.mozilla.org/en-US/docs/Web/API/window.crypto.getRandomValues
                 */

                crypto.getRandomValues(bytes);
                return bytes;
            }

            throw new Error("Secure random number generation not supported by this browser " +
                            "use chrome, FireFox or Internet Explorer 11");
        },

        XOR: function (a, b) {
            var i,
                res = [];

            for (i = 0; i < Math.min(a.length, b.length); i++) {
                res.push(a[i] ^ b[i]);
            }

            return new Uint8Array(res);
        },

        H: function (data) {
            var shaObj;

            shaObj = new JSSHA("SHA-1", "BYTES");
            shaObj.update(ua2text(data));
            return text2ua(shaObj.getHash("BYTES"));
        },

        HMAC: function (key, msg) {
            var shaObj;

            shaObj = new JSSHA("SHA-1", "TEXT");
            shaObj.setHMACKey(ua2text(key), "BYTES");
            shaObj.update(msg);
            return text2ua(shaObj.getHMAC("BYTES"));
        },

        Hi: function (text, salt, iterations) {
            var ui, ui1, i, shaObj, saltBuffer;

            function appendBuffer (buffer1, buffer2) {
                var tmp;

                tmp = new Uint8Array(buffer1.byteLength + buffer2.byteLength);

                tmp.set(new Uint8Array(buffer1), 0);
                tmp.set(new Uint8Array(buffer2), buffer1.byteLength);
                return tmp;
            }

            shaObj = new JSSHA("SHA-1", "BYTES");
            shaObj.setHMACKey(text, "TEXT");

            saltBuffer = appendBuffer(salt, new Uint8Array([0, 0, 0, 1]));

            shaObj.update(String.fromCharCode.apply(null, saltBuffer));
            ui1 = shaObj.getHMAC("BYTES");
            ui = ui1;
            for (i = 0; i < iterations - 1; i++) {
                shaObj = new JSSHA("SHA-1", "BYTES");
                shaObj.setHMACKey(text, "TEXT");
                shaObj.update(ui1);
                ui1 = shaObj.getHMAC("BYTES");
                ui = xor(ui, ui1);
            }

            return text2ua(ui);
        }
    };

    return methods;

});

/* ex:set ts=4 et: */

